Package: nloptr
Type: Package
Title: R interface to NLopt
Version: 1.0.4
Date: 2014-08-02
Author: Jelmer Ypma <uctpjyy@ucl.ac.uk>, with contributions by
        Hans W. Borchers <hwborchers@googlemail.com> and 
        Dirk Eddelbuettel <edd@debian.org>
Maintainer: Jelmer Ypma <uctpjyy@ucl.ac.uk>
Description: 
    nloptr is an R interface to NLopt. NLopt is a free/open-source library for
    nonlinear optimization, providing a common interface for a number of
    different free optimization routines available online as well as original
    implementations of various other algorithms.
    See http://ab-initio.mit.edu/wiki/index.php/NLopt_Introduction for more
    information on the available algorithms. During installation on Unix the
    NLopt code is downloaded and compiled from the NLopt website.
License: LGPL-3
Suggests: testthat (>= 0.8.1)
LazyLoad: yes
Packaged: 2017-08-22 10:59:12 UTC; ripley
NeedsCompilation: yes
Repository: CRAN
Date/Publication: 2017-08-22 13:02:45
Built: R 3.4.1; x86_64-w64-mingw32; 2018-08-02 04:31:32 UTC; windows
Archs: x64
